@@ -2,18 +2,19 @@
 
   if (family=="bernoulli"){
     if (link=="logit"){
-      e <- exp(eta); g <- e/(1+e)
-      ll1  <-  y - g; ll2  <-  -g/(1+e) 
+      e   <- exp(eta); g <- e/(1+e); gg <- g/(1+e)
+      ll1 <- y-g
+      ll2 <- -g/(1+e)
+      ll1.2 <- (-y/e-y+1)*(1+e)
     }
     if (link=="probit"){
       g    <-  pnorm(eta); gg <- g*pnorm(-eta)
-      gg8  <-  pnorm(8)*(1-pnorm(8)) 
-      gg   <-  gg*(gg>gg8) + gg8*(gg<= gg8)  
       g1   <-  dnorm(eta)
       ll1  <-  (y - g)*g1/gg
       g11  <-  g1*g1
       ll2  <-  -eta*g1/gg - (1-2*g)*g11/(gg*gg)
       ll2  <-  ll2*(y-g); ll2 <- ll2 - g11/gg
+      ll1.2 <- (y-g)*g1 /( (y-g)*(-eta*g1-(1-2*g)*g11/gg) -g11)
     }
   }
 
@@ -21,11 +22,13 @@
     if (link=="identity"){
       ll1 <- (y-eta)
       ll2 <- rep(-1,length(eta))
+      ll1.2 <- -y+eta
     }
     if (link=="log"){
       g <- exp(eta)
       ll1 <- (y-g)*g
       ll2 <- (y-2*g)*g
+      ll1.2 <- (y-g)/(y-2*g)
     }
   }
 
@@ -34,6 +37,7 @@
       g <- exp(eta)
       ll1 <- y - g
       ll2 <- -g
+      ll1.2 <- -y/g +1
     }
   }
 
@@ -42,6 +46,7 @@
       g <- 1/eta
       ll1 <- - y + g
       ll2 <- - g*g
+      ll1.2 <- y/(g*g) -1/g
     }
   }
 
@@ -50,6 +55,7 @@
       g <- 1/sqrt(eta)
       ll1 <- 0.5*(-y + g)
       ll2 <- -0.25 *g/eta
+      ll1.2 <- 2*(y*eta/g -eta)
     }
   }
 
@@ -58,10 +64,11 @@
       e <- exp(eta); ee <- 1-e
       ll1 <- y - k*e/ee
       ll2 <- -k/(ee*ee)
+      ll1.2 <- -y*(ee*ee)/k +1
     }
   }
-  
-  l <- data.frame(ll1,ll2)
+
+  l <- data.frame(ll1,ll2,ll1.2)
   return(l)
 }
 
