@@ -1,27 +1,33 @@
 "kreg" <- function(x,y,bandwidth=NULL,grid=TRUE,kernel="biweight",
                    product=TRUE,sort=TRUE){
 
-  p <- 2; q <- 2  ## biweight kernel 
-  if(kernel=="triangle"){                      p <- q <- 1 }    
-  if(kernel=="uniform"){                       p <- 1; q <- 0 } 
-  if(kernel=="epanechnikov"){                  p <- 2; q <- 1 } 
-  if(kernel=="biweight" || kernel=="quartic"){ p <- 2; q <- 2 } 
-  if(kernel=="triweight"){                     p <- 2; q <- 3 } 
-  if(kernel=="gaussian"|| kernel=="normal"){   p <- 0; q <- 0 }
-  
+  if (kernel=="triangular"){ kernel <- "triangle" }
+  if (kernel=="rectangle" || kernel=="rectangular"){ kernel <- "uniform" }
+  if (kernel=="quartic"){ kernel <- "biweight" }
+  if (kernel=="normal"){  kernel <- "gaussian" }
+
+  kernel.names <- c("triangle","uniform","epanechnikov","biweight",
+                    "triweight","gaussian")
+  pp <- c(1,2,2,2,2,0)
+  qq <- c(1,0,1,2,3,NA)
+  names(pp) <- names(qq) <- kernel.names
+
+  p <- pp[kernel]
+  q <- qq[kernel]
+
   x <- as.matrix(x)  ## nxd
   y <- as.matrix(y)  ## nxc
   n <- nrow(x)
   d <- ncol(x)
   c <- ncol(y)
-  
+
   if (sort){
     or <- order(x[,1])
     ro <- order((1:n)[or])
     x  <- x[or,,drop=FALSE]
     y  <- y[or,,drop=FALSE]
   }
-  
+
   if (is.logical(grid)){  ## grid TRUE, but not given
     if (grid){
       ng <- 400
@@ -48,7 +54,7 @@
       grid  <- grid[or.grid,,drop=FALSE]
     }
   }
-  
+
   ##  if (is.null(grid)){
   ##    grid <- x
   ##    m <- n
@@ -63,7 +69,7 @@
   ##      grid  <- grid[org,,drop=FALSE]
   ##    }
   ##  }
-  
+
   if (missing(bandwidth)) {
     if (p > 0){  ## non-gaussian
       fac <- kernel.constants(p,q,d)$d0 / (1/(2*sqrt(pi))^d) ^(1/(d+4))
@@ -74,14 +80,14 @@
     s <- apply(s,2,min)
     bandwidth <- fac *s* n^(-1/(d+4)) ## Scott's ROT
   }
-  
+
   tmp <- convol(x,bandwidth,grid=grid,y=cbind(rep(1,n),y),
                 p=p,q=q,product=product,sort=FALSE)
   mh <- as.matrix((tmp[,2:ncol(tmp)])/(tmp[,1]))
-  
+
   kconst <- kernel.function(t(rep(0,q)), kernel=kernel, product=product)
   df <- n - kconst*sum(1/tmp[,1])/(n*prod(bandwidth))
-  
+
   if (sort){
     return(list(x=grid,y=mh,bandwidth=bandwidth,df=df,rearrange=ro.grid))
   }else{
